//
//  ViewController.swift
//  CloudUploader
//
//  Created by William Sloat on 1/31/20.
//  Copyright © 2020 Aurora Foundry. All rights reserved.
//

import Cocoa
import SwiftyDropbox
import BoxSDK
import AuthenticationServices

class ViewController: NSViewController {

    override func viewDidLoad() {
        super.viewDidLoad()

        // Do any additional setup after loading the view.
        setupDropbox()
    }

    override var representedObject: Any? {
        didSet {
        // Update the view, if already loaded.
        }
    }

    // MARK: Dropbox
    @IBAction func didClickDropboxAuth(_ sender: Any) {
        DropboxClientsManager.authorizeFromController(sharedWorkspace: NSWorkspace.shared,
                                                      controller: nil,
                                                      openURL: { (url: URL) -> Void in
                                                        NSWorkspace.shared.open(url)
                                                    })
    }
    
    
    @IBAction func didClickDropboxAPIRequest(_ sender: Any) {
        // Reference after programmatic auth flow
        let client = DropboxClientsManager.authorizedClient
        
        let fileData = "testing data example".data(using: String.Encoding.utf8, allowLossyConversion: false)!

        client!.files.upload(path: "/CloudUploader/test", input: fileData)
            .response { response, error in
                if let response = response {
                    print(response)
                } else if let error = error {
                    print(error)
                }
            }
            .progress { progressData in
                print(progressData)
            }
    }
    
    func setupDropbox() {
        DropboxClientsManager.setupWithAppKeyDesktop("ik1yxwujngaul2w")
        
        NSAppleEventManager.shared().setEventHandler(self,
                                                     andSelector: #selector(handleGetURLEvent),
                                                     forEventClass: AEEventClass(kInternetEventClass),
                                                     andEventID: AEEventID(kAEGetURL))
    }
    
    @objc func handleGetURLEvent(_ event: NSAppleEventDescriptor?, replyEvent: NSAppleEventDescriptor?) {
        if let aeEventDescriptor = event?.paramDescriptor(forKeyword: AEKeyword(keyDirectObject)) {
            if let urlStr = aeEventDescriptor.stringValue {
                let url = URL(string: urlStr)!
                if let authResult = DropboxClientsManager.handleRedirectURL(url) {
                    switch authResult {
                    case .success:
                        print("Success! User is logged into Dropbox.")
                    case .cancel:
                        print("Authorization flow was manually canceled by user!")
                    case .error(_, let description):
                        print("Error: \(description)")
                    }
                }
                // this brings your application back the foreground on redirect
                NSApp.activate(ignoringOtherApps: true)
            }
        }
    }
    
    // MARK: Box
    
    private var sdk: BoxSDK!
    private var client: BoxClient!
    private var folderItems: [FolderItem] = []
    private let initialPageSize: Int = 100
    
    enum Constants {
        static let clientId = ""
        static let clientSecret = ""
    }
    
    private lazy var dateFormatter: DateFormatter = {
        let formatter = DateFormatter()
        formatter.dateFormat = "MMM dd,yyyy at HH:mm a"
        return formatter
    }()
    
    @IBAction func didClickBoxAuth(_ sender: Any) {
        let sdk = BoxSDK(clientId: Constants.clientId, clientSecret: Constants.clientSecret)
        sdk.getOAuth2Client() { result in
            switch result {
            case let .success(client):
                print("Use client to make API calls")
            case let .failure(error):
                print("Handle error creating client")
            }
        }
    }
    
    
    @IBAction func didClickBoxAPIRequest(_ sender: Any) {

    }
}

